/**************************************************************************
 **************************************************************************
 
  Program:  Gstep_TWI_Functions		Date: December 29, 2008
 
  Author:  S. Swift		    		Version: 1.77
  
  Copyright 2002-2009 SOC Robotics, Inc.
 
  TWI Functions for handling peripherals such as Motors, DRO's, joysticks
  etc.
		 
 ***************************************************************************	 
 ***************************************************************************/
#include <iom644v.h>
#include <macros.h>
#include <ctype.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <eeprom.h>
#include <stdio.h>

//#include "uart.h"
#include "Gstep_Defines.h"
#include "Gstep_Extern.h" 
#include "gstep_ExternF.h"

// ***********************************************************************
// ***********************************************************************
//     Start of Code
// ***********************************************************************
// ***********************************************************************


// Update status of Joystick LEDs - handle CJ14 and CJ34
void JoyStickLED(char led,char state)
{
 	 if(CJ34present==YES) {
	     if(led==JOYGREENLED) {
	         if(state==ON) Set_SmartJoystick_Leds(SMARTJOYSTICKTWIADDRESS,SMARTLED1,SMARTLEDON);
		     else Set_SmartJoystick_Leds(SMARTJOYSTICKTWIADDRESS,SMARTLED1,SMARTLEDOFF);
		     }
	     else {
	         if(state==ON) Set_SmartJoystick_Leds(SMARTJOYSTICKTWIADDRESS,SMARTLED2,SMARTLEDON);
		     else Set_SmartJoystick_Leds(SMARTJOYSTICKTWIADDRESS,SMARTLED2,SMARTLEDOFF);
		     }
	     
	     }
 	 else {
	     if(led==JOYGREENLED) {
	         if(state==ON) ClrBit(PORTA,PINA6);
		     else SetBit(PORTA,PINA6);
		     }
	     else {
	         if(state==ON) ClrBit(PORTA,PINA7);
		     else SetBit(PORTA,PINA7);
		     }
		 }
}

const char joycalmes1[] = "\n\rCalibrate joystick - press 'e' to exit\n\r";
const char joycalmes2[] = "Analog [x,y,z,a]: ";
const char joycalmes3[] = "1               ";
const char joycalmes4[] = "0                ";

// Calibrate joystick by displaying values then load into EEPROM
void Calibrate_Joystick()
{
    putfs(joycalmes1);
    while((getchar()!='e')) {
	    putfs(joycalmes2);
	    printf("%d ",analogch0);
		printf("%d ",analogch1);
		printf("%d ",analogch2);
		printf("%d ",analogch3);	
		if((PINA&0x10)) printf("1 ");
		else printf("0 ");
		if((PINA&0x20)) putfs(joycalmes3);
		else putfs(joycalmes4);
		putchar(CR);
		}
	anch0center = analogch0;
	anch1center = analogch1;
	anch2center = analogch2;
	anch3center = analogch3;
	SaveEepromPreferences();
}

// Configure LED8 display device via TWI to accept float with
// decimalpt position set
void  Init_LED8_TWI(unsigned char twiaddress,int decimalpt)
{
	 void *intvalue;

	 messageBuf[0] = 6;
	 intvalue = &messageBuf[1];
	 *(int*)intvalue = decimalpt;
	 TWI_Start_Write(twiaddress,messageBuf,3);
}

// Write float value to LED8 via TWI
void Write_Float_LED8(unsigned char twiaddress,float fvalue)
{
 	 void *value;
	 
	 messageBuf[0] = 3;
	 value = &messageBuf[1];
	 *(float *)value = fvalue;
	 TWI_Start_Write(twiaddress,messageBuf,5);
}

unsigned char Read_LED8_Switches(char twiaddress)
{
	TWI_Start_Read(twiaddress,3);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

#define	   TWI_MTX_ADR_NACK  0x20

// Check existance of LED8's: 0x20, 0x21, 0x22, 0x24
void Enumerate_LED8()
{
	 led8state = 0;
	 
	 // Check for X axis LED8
	 Init_LED8_TWI(LED8XAXIS,LEDFIVEDECIMAL);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 led8state |= 0x01;
	 	 delaymsec(10);
	 	 Write_Float_LED8(LED8XAXIS,0.0);
		 }
//	else printf("\n\rX axis LED8 not found\n\r");
 
	 // Check for y axis LED8
	 Init_LED8_TWI(LED8YAXIS,LEDFIVEDECIMAL);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	     led8state |= 0x02;
	 	 delaymsec(10);
	 	 Write_Float_LED8(LED8YAXIS,0.0);
		 }
//	 else printf("\n\rY axis LED8 not found\n\r");

	 // Check for z axis LED8
	 Init_LED8_TWI(LED8ZAXIS,LEDFIVEDECIMAL);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 led8state |= 0x04;
	 	 delaymsec(10);
	 	 Write_Float_LED8(LED8ZAXIS,0.0);
		 }
//	 else printf("\n\rZ axis LED8 not found\n\r");
	
	 // Check for a axis LED8
	 Init_LED8_TWI(LED8AAXIS,LEDFIVEDECIMAL);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	     led8state |= 0x08;
	 	 delaymsec(10);
	 	 Write_Float_LED8(LED8AAXIS,0.0);
		 }
//	 else printf("\n\rA axis LED8 not found\n\r");
}

void Display_LED84()
{
    float xpos,ypos,zpos,apos;
	
    if((led8state&0x01)) {
	    xpos = ((float)xoldpos)*xresolution;
	    if(xdirpolarity==NEGATIVE)  xpos = -xpos;
	    Write_Float_LED8(LED8XAXIS,xpos);
		}
					  
	if((led8state&0x02)) {
		ypos = ((float)yoldpos)*yresolution;
		if(ydirpolarity==NEGATIVE)  ypos = -ypos;
	 	Write_Float_LED8(LED8YAXIS,ypos);
		}

	if((led8state&0x04)) {
		zpos = ((float)zoldpos)*zresolution;
		if(zdirpolarity==NEGATIVE)  zpos = -zpos;
	 	Write_Float_LED8(LED8ZAXIS,zpos);
		}
					  
	if((led8state&0x08)) {
		apos = ((float)holdpos)*aresolution;
		if(adirpolarity==NEGATIVE)  apos = -apos;
	 	Write_Float_LED8(LED8AAXIS,apos);
		}
}

// Configure SmartLCD display device via TWI to accept float
void  Init_SmartLCD_TWI(unsigned char twiaddress)
{
	 messageBuf[0] = 'm';
	 messageBuf[1] = 'd';
	 TWI_Start_Write(twiaddress,messageBuf,2);
}

// Write float value to LED8 via TWI
void Write_Float_SmartLCD(unsigned char twiaddress,unsigned char axisid,float fvalue)
{
 	 void *value;
	 
	 messageBuf[0] = 'm';
	 messageBuf[1] = axisid;
	 value = &messageBuf[2];
	 *(float *)value = fvalue;
	 TWI_Start_Write(twiaddress,messageBuf,6);
}

unsigned char Read_SmartLCD_Switches(char twiaddress)
{
	TWI_Start_Read(twiaddress,3);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

void Display_SmartLCD()
{
    float xpos,ypos,zpos,apos;
	
	xpos = ((float)xoldpos)*xresolution;
	if(xdirpolarity==NEGATIVE)  xpos = -xpos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'x',xpos);
	delaymsec(4);
	
	ypos = ((float)yoldpos)*yresolution;
	if(ydirpolarity==NEGATIVE)  ypos = -ypos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'y',ypos);
	delaymsec(4);

	zpos = ((float)zoldpos)*zresolution;
	if(zdirpolarity==NEGATIVE)  zpos = -zpos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'z',zpos);
	delaymsec(4);
					  
	apos = ((float)holdpos)*aresolution;
	if(adirpolarity==NEGATIVE)  apos = -apos;
	Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'a',apos);
	//delaymsec(5);
}

void Enumerate_SmartLCD()
{
 	 smartlcdstate = 0;
	 
	 // Check for Smart Joystick
	 delaymsec(50);
	 Init_SmartLCD_TWI(SMARTLCDTWIADDRESS);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 smartlcdstate |= 0x01;
	 	 delaymsec(50);
	 	 Display_SmartLCD();
		 }
}

// Configure SmartJoystick device via TWI to accept float with
// decimalpt position set
void  Init_SmartJoystick_TWI(unsigned char twiaddress)
{
	 messageBuf[0] = 'r';
	 TWI_Start_Write(twiaddress,messageBuf,1);
}

unsigned char Read_SmartJoystick_Switches(unsigned char twiaddress)
{
	TWI_Start_Read(twiaddress,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

// Get the the step mode of a Motor Controller
void Set_SmartJoystick_Leds(unsigned char twiaddress,
                            unsigned char ledid,unsigned char state)
{
	messageBuf[0] = 's';
	messageBuf[1] = 'l';
	messageBuf[2] = ledid;
	messageBuf[3] = state;
	TWI_Start_Write(twiaddress,messageBuf,4);
}

// Get the the step mode of a Motor Controller
void Set_SmartJoystick_Calibrate(unsigned char twiaddress)
{
	messageBuf[0] = 's';
	messageBuf[1] = 'c';
	TWI_Start_Write(twiaddress,messageBuf,2);
}

// Get Smart Joystick joystick values
void Get_SmartJoystick_Analogchs(unsigned char twiaddress)
{
	messageBuf[0] = 'g';
	messageBuf[1] = 'a';
	TWI_Start_Write(twiaddress,messageBuf,2);
	delaymsec(25);	
	TWI_Start_Read(twiaddress,5);
    TWI_Get_Data_From_Transceiver(messageBuf,5);
	smartanalogch0 = messageBuf[0];
	smartanalogch1 = messageBuf[1];
	smartanalogch2 = messageBuf[2];
	smartanalogch3 = messageBuf[3];
	smartjoyswitches = messageBuf[4];
//	printf("smartanch= %d %d %d %d\n\r",smartanalogch0,smartanalogch1,
//					   	  	 			smartanalogch2,smartanalogch3);
}

// Get Smart Joystick Joystick center values
void Get_SmartJoystick_Center(unsigned char twiaddress)
{
	messageBuf[0] = 'g';
	messageBuf[1] = 'c';
	TWI_Start_Write(twiaddress,messageBuf,2);
	delaymsec(25);	
	TWI_Start_Read(twiaddress,4);
    TWI_Get_Data_From_Transceiver(messageBuf,4);
	smartanch0center = messageBuf[0];
	smartanch1center = messageBuf[1];
	smartanch2center = messageBuf[2];
	smartanch3center = messageBuf[3];
//	printf("smartcenter= %d %d %d %d\n\r",smartanch0center,smartanch1center,
//					   	  	 			smartanch2center,smartanch3center);
}

void Enumerate_SmartJoystick()
{
	 // Check for Smart Joystick
	 Init_SmartJoystick_TWI(SMARTJOYSTICKTWIADDRESS);
     if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	 CJ34present |= 0x01;
	 	 delaymsec(200);
		 JoyStickLED(JOYGREENLED,OFF);
		 delaymsec(25);
		 JoyStickLED(JOYREDLED,OFF);
		 delaymsec(25);
		 // Re-calibrate Smart Joystick center
		 Set_SmartJoystick_Calibrate(SMARTJOYSTICKTWIADDRESS);
		 delaymsec(250);
		 // Load Smart Joystick center values
		 Get_SmartJoystick_Center(SMARTJOYSTICKTWIADDRESS);
		 delaymsec(25);
		 Get_SmartJoystick_Analogchs(SMARTJOYSTICKTWIADDRESS);
		 delaymsec(25);
		 }
}

// Set TWI step mode - wave, full, half
void  Set_Motor_TWI_StepMode(unsigned char twiaddress,unsigned char mode)
{
 	messageBuf[0] = 's';
	messageBuf[1] = mode;
	TWI_Start_Write(twiaddress,messageBuf,2);  
}

// Get the the step mode of a Motor Controller
unsigned char Get_Motor_TWI_StepMode(unsigned char twiaddress)
{
	messageBuf[0] = 'g';
	messageBuf[1] = 's';
	TWI_Start_Write(twiaddress,messageBuf,2);
	delaymsec(15);	
	TWI_Start_Read(twiaddress,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
	return messageBuf[0];
}

// Send the reset command to each motor controller and verify
// it exists
void Init_TWI_MotorControllers()
{
 	 unsigned char i,motormask=0x01;
	 
	 messageBuf[0] = 'r';
	 for(i=0; i<8; i++)  {
	     TWI_Start_Write((MOTORCONTROLLER_X_TWIADDRESS+i),messageBuf,1);
         if(TWI_Get_State_Info()!=TWI_MTX_ADR_NACK) {
	 	     motorcontrollerstate |= motormask;
	 	     delaymsec(5);
		     // TODO - configure motor controller state
		     }
		 motormask <<= 1;
	 	 delaymsec(5);
		 }
}

unsigned char Read_MotorController_Status(char twiaddress)
{
	TWI_Start_Read(twiaddress,1);
    TWI_Get_Data_From_Transceiver(messageBuf,1);
    return messageBuf[0];
}

//  Setup Motor controllers to default states
void Enumerate_MotorControllers()
{
 	 motorcontrollerstate = 0x00;
	 Init_TWI_MotorControllers();
}

void PrintMotorAxisChar(axisid)
{
 	 if(axisid==0) putchar('X');
	 else if(axisid==1) putchar('Y');
	 else if(axisid==2) putchar('Z');
	 else if(axisid==3) putchar('A');
	 else if(axisid==4) putchar('D');
	 else if(axisid==5) putchar('E');
	 else if(axisid==6) putchar('F');
	 else if(axisid==7) putchar('G');
}

const char enummes1[] = "\n\rEnumerate TWI Devices\n\r";
const char enummes11[] = " Motor Controllers for ";
const char enummes2[] = " No Motor Controllers found\n\r";
const char enummes3[] = " found\n\r";
const char enummes4[] = " DRO found with ";
const char enummes45[] =" No DRO found\n\r";
const char enummes5[] = " present\n\r";
const char enummes6[] = " No Smart Joystick found\n\r";
const char enummes7[] = " Smart Joystick found - disabled\n\r";
const char enummes8[] = " No SmartLCD found\n\r";
const char enummes9[] = " SmartLCD found - enabled\n\r";

void Enumerate_TWI_Devices()
{
 	 unsigned char i,motormask=0x01;
	 
	 // Enumerate Motor Controllers
	 putfs(enummes1);
	 delaymsec(1500);
	 Enumerate_MotorControllers();
	 delaymsec(15);
	 if(motorcontrollerstate==0) putfs(enummes2);
	 else {
	     putfs(enummes11);
		 for(i=0; i<8; i++) {
	         if((motorcontrollerstate&motormask)) {
		         PrintMotorAxisChar(i);
			     printf("-%c ",Get_Motor_TWI_StepMode(MOTORCONTROLLER_X_TWIADDRESS+i));
			     }
			 motormask <<= 1;
			 }
	 	 putfs(enummes3);
		 }
		 		 
	 // Enumerate DRO's
	 delaymsec(2000);
	 Enumerate_LED8();
	 if(led8state==0) putfs(enummes45);
	 else {
	     putfs(enummes4);
	     if((led8state&0x01)) putchar('X');
	 	 if((led8state&0x02)) putchar('Y');
	 	 if((led8state&0x04)) putchar('Z');
	 	 if((led8state&0x08)) putchar('A');
	 	 putfs(enummes5);
		 }
	
	 // Enumerate Smart Joystick
	 delaymsec(25);
	 Enumerate_SmartJoystick();
	 if(CJ34present==NO) putfs(enummes6);
	 else putfs(enummes7);
	 
	 // Enumerate SmartLCD
	 delaymsec(25);
	 Enumerate_SmartLCD();
	 if(smartlcdstate==0) putfs(enummes8);
	 else putfs(enummes9);
}

// Check the status of LED8 joystick buttons and move axis accordingly
void Follow_LED_Joystick()
{
 	unsigned char switchstate,gotonewpos,gotonewzpos,gotonewapos;
	
	gmode = G1;
			   
	// Use default feed rate of 5.0ips
	interpolatemode = ON;
	
	gotonewpos = gotonewzpos = gotonewapos = 0;
	if((led8state&0x01)&&(led8joystate&0x01)) {
		switchstate = Read_LED8_Switches(LED8XAXIS);
		if(switchstate==1) xnewpos = xoldpos + 1;
		if(switchstate==2) xnewpos = xoldpos - 1;
		if(switchstate) {
		    gotonewpos = 1;
			//xoldpos = xnewpos;
			}
		}
	if((led8state&0x02)&&(led8joystate&0x02)) {
		switchstate = Read_LED8_Switches(LED8YAXIS);
		if(switchstate==1) ynewpos = yoldpos + 1;
		if(switchstate==2) ynewpos = yoldpos - 1;
		if(switchstate) {
		    gotonewpos = 1;
			//yoldpos = ynewpos;
			}
		}
	if((led8state&0x04)&&(led8joystate&0x04)) {
		switchstate = Read_LED8_Switches(LED8ZAXIS);
		if(switchstate==1) znewpos = zoldpos + 1;
		if(switchstate==2) znewpos = zoldpos - 1;
		if(switchstate) {
		    gotonewzpos = 1;
			//zoldpos = znewpos;
			}
		}
	if((led8state&0x08)&&(led8joystate&0x08)) {
		switchstate = Read_LED8_Switches(LED8AAXIS);
		if(switchstate==1) hnewpos = holdpos + 1;
		if(switchstate==2) hnewpos = holdpos - 1;
		if(switchstate) {
		    gotonewapos = 1;
			//holdpos = hnewpos;
			}
		}
		
	// Run stepper motors if any LED8 joystick switchs activated
	if((gotonewpos==1)||(gotonewzpos==1)||(gotonewapos==1)) {
	    // Move the x,y,z under LED8 joystick control
	    // Need to change this so 3D movement is possible
	    //  mill_3dline();
		//printf("\n\r execute move");
	    if(gotonewpos) mill_line();  // change to mill 3D line?
	    if(gotonewzpos) {
		    update_z();
		    runsteppers();
		    }
	    if(gotonewapos) {
		    update_h();
		    runsteppers();
		    }
	    // For fine control delay time to process next command
	    delaymsec(75);
		}
			   		   
	interpolatemode = OFF;
	delayvalue = feedrate;
}


const char TWImess1[] = "\n\rTWI Interactive Command Mode\n\r";
const char TWImess2[] = "Enter twi address (hex-two chars) followed by ASCII commands\n\r";
const char TWImess3[] = "  Example:  14xtx    Enter 'e' to exit\n\r";

// Send commands on I2C bus given twi address and an ASCII string
//  TBD - get data command
void Execute_TWI_Command()
{ 
     unsigned char twiaddress,iptr;
	  
	 putfs(TWImess1);
	 putfs(TWImess2);
	 putfs(TWImess3);
	 putchar(':');
 	 while((instrg[0]=tolower(getc()))!='e') {
	     instrg[1] = getc();
		 instrg[2] = 0;
		 sscanf(instrg,"%x",&twiaddress);
		 iptr = 0;
	     while((messageBuf[iptr++]=getc())!=CR) {
		     if(iptr==20) break;
			 }
		 messageBuf[iptr] = 0;
		 
		 //printf("\n\rtwi=%x %s",twiaddress,messageBuf);
		 
		 TWI_Start_Write(twiaddress,messageBuf,iptr);

	     putCRLF(); putchar(':');
	     }
	 putCRLF();
}

